package de.cnc.domainparser;


import de.cnc.domainparser.util.DomainParserUtil;
import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert den Bereich zwischen
 * ffnender und schliessender eckiger Klammer [ ... ] .
 */
public class BracketArea extends AbstractNonTerminalElement
{
    private CommaSeparatedList commaSeparatedList;

    private final String completeSrcStr;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private BracketArea(SourcePosition pSrcPos, String pSrcStr)
    {
        // Anfangsposition vermerken
        this.srcPos = pSrcPos.getClone();

        this.completeSrcStr = pSrcStr;
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static BracketArea parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        if (pStr.charAt(pSrcPos.getPos()) != '[')
        {
            return null; // oder exception werfen
        }

        BracketArea bracketArea = new BracketArea(pSrcPos, pStr);
        
        // Vermerken der Anfangsposition
        int anfPos = pSrcPos.getPos();        

        pSrcPos.addOne(); // ffnende Klammer berspringen

        // System.out.println( "ParenthesisArea#pSrcPos: " + pSrcPos );


        // nach der ffnenden Klammer folgende WhiteSpaces berspringen
        DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

        if (pStr.charAt(pSrcPos.getPos()) == ']')
        {
            // auf die ffnende Klammer folgt unmittelbar eine schliessende Klammer
            pSrcPos.addOne(); // schliessende Klammer berspringen

            // Lnge vermerken
            bracketArea.length = pSrcPos.getPos() - anfPos;

            return bracketArea;
        }

        // System.out.println( "ParenthesisArea#parenArea.commaSeparatedList: " + parenArea.commaSeparatedList );
        CommaSeparatedList tmpCommaSeparatedList = CommaSeparatedList.parse(pStr, pSrcPos,
                pParserContext);

        bracketArea.commaSeparatedList = tmpCommaSeparatedList;

        // nach dem Element folgende WhiteSpaces berspringen
        DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

        // char runChar = pStr.charAt(pSrcPos.getPos());
        // System.out.println( "ParenthesisArea#runChar: " + runChar ) ;

        if (pSrcPos.getPos() >= pStr.length())
        {
            // keine schliesende Klammer aufgetreten
            throw new DomainParseException("right bracket expected ']', end of source reached", pSrcPos);
        }
        else if (pStr.charAt(pSrcPos.getPos()) != ']')
        {
            // keine schliesende Klammer aufgetreten
            throw new DomainParseException("right bracket expected ']'", pSrcPos);
        }

        bracketArea.endSourcePosition = pSrcPos.getClone();

        pSrcPos.addOne(); // schliessende Klammer berspringen

        // runChar = pStr.charAt(pSrcPos.getPos());
        // System.out.println( "ParenthesisArea#runChar: " + runChar ) ;

        // Lnge vermerken
        bracketArea.length = pSrcPos.getPos() - anfPos;

        return bracketArea;
    }

    /**
     * 
     * @param pI
     * @return
     */
    public CommaSeparatedList getCommaSeparatedList()
    {
        return this.commaSeparatedList;
    }

    /**
     * 
     */
    public String toString()
    {
        return toString(0);
    }

    /**
     * 
     * @param indent
     * @return
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        if (this.commaSeparatedList == null)
        {
            strBuff.append(StringUtil.spaces(indent + 1) + "is empty");
        }
        else
        {
            strBuff.append(this.commaSeparatedList.toString(indent + 1));
        }
        return strBuff.toString();
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        if (this.commaSeparatedList == null)
        {
            return true;
        }
        return this.commaSeparatedList.isEmpty();
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        // throw new UnsupportedOperationException();
        return this.completeSrcStr.substring(this.srcPos.getPos(), this.srcPos.getPos() + this.length + 2);
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        // throw new UnsupportedOperationException();
        return this.completeSrcStr.substring(this.srcPos.getPos() + 1, this.srcPos.getPos() + this.length + 1);
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return getContent().indexOf("\n") >= 0;
    }

    /**
     * Lschen Komentare
     */
    public void stripComments(boolean pAllComments)
    {
        if (this.commaSeparatedList != null)
        {
            this.commaSeparatedList.stripComments(pAllComments);
        }
    }
}
