package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;


/**
 * Diese Klasse reprsentiert einen Bracket-Kommentar / * xxx * /
 */
public class BracketComment extends AbstractTerminalElement
{

    private String content;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private BracketComment(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        this.srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static BracketComment parse(
            final String pStr, 
            final SourcePosition pSrcPos) throws DomainParseException
    {
        String partStr = pStr.substring(pSrcPos.getPos());

        if (!partStr.startsWith("/" + "*"))
        {
            return null; // oder exception werfen
        }

        BracketComment brcComment = new BracketComment(pSrcPos);

        // ffnende Klammerung berspringen
        pSrcPos.addOne();
        pSrcPos.addOne(); 

        StringBuffer strBuff = new StringBuffer();

        while (pSrcPos.getPos() < pStr.length() - 1 
                && !("*" + "/").equals(pStr.substring(pSrcPos.getPos(), pSrcPos.getPos() + 2)))
        {
            char runChar = pStr.charAt(pSrcPos.getPos());

            if (runChar == '\n') // NewLine
            {
                pSrcPos.addNewLine();
            }
            else
            {
                pSrcPos.addOne();
            }
            strBuff.append(runChar);
        }

        if (pSrcPos.getPos() >= pStr.length() - 1)
        {
            // keine schliesende Klammerung aufgetreten
            throw new DomainParseException("right bracket comment parenthesis expected '*" + "/'", pSrcPos);
        }

        // schliessende Klammerung berspringen
        pSrcPos.addOne();
        pSrcPos.addOne();
        brcComment.content = strBuff.toString();        
        return brcComment;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return this.content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        return strBuff.toString() + " content: " + this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return "/" + "*" + this.content + "*" + "/";
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return this.content.trim().length() < 1;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return getContent().indexOf("\n") >= 0;
    }

    /**
     * Prfen, ob es sich um einen Javadoc-Kommentar handelt..
     * 
     * @return ob es sich um einen Javadoc-Kommentar handelt
     */
    public boolean isJavadoc()
    {
        return getContent().length() > 4 && getContent().charAt(2) == '*';
    }

    /**
     * Abfragen inneren Teil eines Javadoc-Kommentars.
     * Noch nicht fr mehrzeilige Javadoc's fertig programmiert.
     * 
     * @return
     */
    public String getJavadocContent()
    {
        if (! isJavadoc())
        {
            throw new RuntimeException("no javadoc comment " + getSourcePos());
        }
        //bei einzeiligem Kommentar
        return getContent().substring(3, getContent().length() - 2).trim();
    }

}
