package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert ein String-Literal in Apostrophen eingeschlossen.
 * "..."
 */
public class HighCommaStringLiteral extends AbstractTerminalElement
{
    private String content;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private HighCommaStringLiteral(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        this.srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static HighCommaStringLiteral parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {

        if (pStr.charAt(pSrcPos.getPos()) != '\'')
        {
            return null; // oder exception werfen
        }
        if (pParserContext.isJavaStringEncodingEnabled)
        {
            throw new UnsupportedOperationException("noch nicht fertig programmiert");
        }

        
        HighCommaStringLiteral qStrLit = new HighCommaStringLiteral(pSrcPos);

        // den ersten Apostroph berspringen
        pSrcPos.addOne();

        StringBuffer strBuff = new StringBuffer();

        while (pSrcPos.getPos() < pStr.length())
        {
            char runChar = pStr.charAt(pSrcPos.getPos());

            if ( pParserContext.isJavaStringEncodingEnabled //--
                    && runChar == '\\' )
            {
                //verarbeiten von java-encodet Strings
                pSrcPos.addOne();
                final char encodedChar = pStr.charAt( pSrcPos.getPos() );
                strBuff.append( StringUtil.getDecodedChr( encodedChar ) );
            }
            else
            {
                if (runChar == '\'')
                {
                    break;
                }
                if ( ! pParserContext.isNewlineInStringEnabled //--
                       && ( runChar == '\n' || runChar == '\r' ) )
                {
                    throw new DomainParseException("end of high comma string literal expected, end of line found",
                            pSrcPos);
                }
                if ( runChar == '\r' //--
                    && DomainParserContext.lineSeparator.startsWith( "\r" ) )
                {
                    // carriage return auf win-systemen berspringen
                    continue;
                }

                strBuff.append(runChar);
            }
            pSrcPos.addOne();
        }
        // den letzten Apostroph berspringen
        pSrcPos.addOne();

        qStrLit.content = strBuff.toString();        

        return qStrLit;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return this.content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        return strBuff.toString() + " content: " + this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return "'" + this.content + "'";
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return this.content.trim().length() < 1;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return false;
    }

}
