package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert einen Java-Identifier (Bezeichner).
 * Im Bezeichner ist ein Punkt erlaubt. (spter optional machen)
 * Punkt erlaubt wieder ausgebaut wegen ComplexIdentifier
 */
public class Identifier extends AbstractTerminalElement
{
    private String content;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private Identifier(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        this.srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static Identifier parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
    {
        if (pSrcPos.getPos() >= pStr.length()
                || !Character.isJavaIdentifierStart(pStr.charAt(pSrcPos.getPos())))
        {
            return null; // oder exception werfen
        }
        
        Identifier identifier = new Identifier(pSrcPos);

        StringBuffer strBuff = new StringBuffer();

        while (pSrcPos.getPos() < pStr.length())
        {
            char runChar = pStr.charAt(pSrcPos.getPos());

            if (pParserContext.isPointInIdentifierAllowed && runChar == '.' // Punkt im Bezeichner erlaubt
                    && pSrcPos.getPos() < (pStr.length() - 1)
                    && Character.isJavaIdentifierStart(pStr.charAt(pSrcPos.getPos() + 1)))
            {
                ;
            }
            else if (pParserContext.isColonInIdentifierAllowed && runChar == ':' // Doppel-Punkt im Bezeichner erlaubt
                && pSrcPos.getPos() < (pStr.length() - 1)
                && Character.isJavaIdentifierStart(pStr.charAt(pSrcPos.getPos() + 1)))
            {
                ;
            }
            else if (!Character.isJavaIdentifierPart(runChar))
            {
                break;
            }
            strBuff.append(runChar);
            pSrcPos.addOne();
        }
        identifier.content = strBuff.toString();        
        return identifier;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return this.content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        return strBuff.toString() + StringUtil.spaces(indent + 1) + " content: " + this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return false;
    }

}
