package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert einen Operator
 *
 * noch zuschaltbare Operatoren einbauen (nach Lnge sortieren)
 *
 * noch Operator-Prioritt einbauen
 */
public class Operator extends AbstractTerminalElement
{

    // ------------------------------------------------------------
    // !!! die Operatoren mssen nach der Lnge sortiert sein
    // !!! die lngsten zuerst
    // !!! sonst knnte bei += ein + erkannt werden
    // ------------------------------------------------------------

    static final String[] OPERATOR_VARIANTS = new String[] {
        "==", //-
        "+=", //-
        "-=", //-
        "*=", //-
        "/=", //-
        ":=", //-
        "!=", //-
        "<>", //-
        "<=", //-
        "=<", //-
        ">=", //-
        "=>", //-
        "&&", //-
        "||", //-
        "=",  //-
        ">",  //-
        "<",  //-
        "+",  //-
        "-",  //-
        "*",  //-
        "/",  //-
        "%",  // %
        "$",  // $
        "&",  // and
        "|",  // or
        "!",  // not
        // momentan fr session:varname in cnc ausser betrieb: ":",  // :
        ".",  // .
        "#"   // #
    };

    private String content;

    /**
     * Konstruktor
     *
     * @param pSrcPos
     */
    private Operator(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        this.srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static Operator parse(
            final String pStr,
            final SourcePosition pSrcPos) throws DomainParseException // Achtung Seiteneffekt
    {
        String partStr = pStr.substring(pSrcPos.getPos());

        for (int i = 0; i < OPERATOR_VARIANTS.length; i++)
        {
            if (partStr.startsWith(OPERATOR_VARIANTS[ i ]))
            {
                Operator operator = new Operator(pSrcPos);

                operator.content = OPERATOR_VARIANTS[ i ];
                pSrcPos.add(OPERATOR_VARIANTS[ i ].length());
                return operator;
            }
        }
        // no operator recognized
        return null;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return this.content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        return strBuff.toString() + StringUtil.spaces(indent) + " content: " + this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return this.content;
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return false;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return false;
    }

}
