package de.cnc.domainparser;


import java.util.ArrayList;
import java.util.Iterator;

import de.cnc.domainparser.util.DomainParserUtil;
import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert eine eingelesene Datei.
 */
public class Unit
{
    String strSrc;

    int srcPos;
    
    private ArrayList segmentList = new ArrayList();
    
    private final String completeSrcStr;
    
    private String fileName;

    /**
     * @return Returns the fileName.
     */
    public String getFileName()
    {
        return this.fileName;
    }
    /**
     * @param pFileName The fileName to set.
     */
    public void setFileName(String pFileName)
    {
        this.fileName = pFileName;
    }
    /**
     * @param pStr
     */
    private Unit(String pStr, String pSrcStr)
    {
        this.strSrc = pStr;

        // gesamten Quelltext referenzieren
        this.completeSrcStr = pSrcStr;
    }

    public static Unit parseFile( 
            final String pFileName, //--
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        final String defFileStr = StringUtil.file2str(pFileName);
        return parseStr(defFileStr, pFileName, pParserContext);
    }

    
    public static Unit parseStr( 
            final String pStr,
            final String pFileName, //--
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        final Unit unit = new Unit(pStr, pStr);
        unit.setFileName(pFileName);
        final SourcePosition srcPos = new SourcePosition(pFileName);

        // am Anfang stehende WhiteSpaces berspringen
        DomainParserUtil.parseWhitesAndComments(pStr, srcPos);

        while (srcPos.getPos() < pStr.length())
        {
            // Segment parsen
            Segment segment = Segment.parse(pStr, srcPos, pParserContext);

            // System.out.println( "Unit#segment " + segment );

            // Segment vermerken
            unit.segmentList.add(segment);

            // nach dem Segment folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, srcPos);
        }
        return unit;
    }

    /**
     * 
     * @return
     */
    public int segmentCount()
    {
        return this.segmentList.size();
    }

    /**
     * 
     * @param pI
     * @return
     */
    public Segment getSegment(int pI)
    {
        return (Segment) this.segmentList.get(pI);
    }

    /**
     * 
     */
    public String toString()
    {
        StringBuffer strBuff = new StringBuffer();

        strBuff.append(StringUtil.getShortClassName(this) + "\n");
        for (int i = 0; i < this.segmentList.size(); i++)
        {
            strBuff.append(((Segment) this.segmentList.get(i)).toString(1)/* + "\n" */);
        }
        return strBuff.toString();
    }

    public String getContent()
    {
        return this.completeSrcStr;
    }

    /**
     * Lschen Komentare
     */
    public void stripComments(final boolean pAllComments)
    {
        for (Iterator iter = this.segmentList.iterator(); iter.hasNext();)
        {
            Object o = iter.next();

            if (o instanceof InlineComment)
            {
                iter.remove();
            }
            else if (o instanceof BracketComment 
                && ( pAllComments || ! ( (BracketComment) o ).isJavadoc() ) )
            {
                iter.remove();
            }
            else if (!((AbstractElement) o).isTerminal())
            {
                ((AbstractElement) o).stripComments(pAllComments);
            }
            
        }
    }

    /**
     * Lschen leere Segmente
     */
    public void stripEmptySegments()
    {
        for (Iterator iter = this.segmentList.iterator(); iter.hasNext();)
        {
            Segment segment = (Segment) iter.next();

            if ( segment.isEmpty() )
            {
                iter.remove();
            }
            else
            {
                segment.stripEmptySegments();
            }
        }
    }

}
