package de.cnc.domainparser.demo;


import de.cnc.domainparser.AbstractElement;
import de.cnc.domainparser.BracesArea;
import de.cnc.domainparser.BracketArea;
import de.cnc.domainparser.BracketComment;
import de.cnc.domainparser.CommaSeparatedList;
import de.cnc.domainparser.ElementList;
import de.cnc.domainparser.InlineComment;
import de.cnc.domainparser.ParenthesisArea;
import de.cnc.domainparser.Segment;
import de.cnc.domainparser.Unit;
import de.cnc.util.StringUtil;


public class Beautifier
{

    /**
     * 
     */
    private Beautifier()
    {
        super();
    }
    
    public static String format(Unit pUnit)
    {
        return format(pUnit, 0);
    }

    public static String format(final Unit pUnit, final int indent)
    {
        StringBuffer strBuff = new StringBuffer();

        for (int i = 0; i < pUnit.segmentCount(); i++)
        {
            Segment segment = pUnit.getSegment(i);

            format(segment, strBuff, indent);
            if (!segment.isBracesSegment())
            {
                // kein Braces-Segment (geschwungene Klammern)
                // Semikolon als Trenner
                strBuff.append(";");
            }
            // else
            // {
            // //geschwungene Klammern anfgen
            // strBuff = new StringBuffer( "\n" + StringUtil.spaces( indent ) + "{\n" 
            // + strBuff + StringUtil.spaces( indent ) + "}\n" ) ;
            // }
            strBuff.append("\n\n");
        }
        return strBuff.toString();
    }

    /**
     * @param segment
     * @param i
     * @return
     */
    private static void format(
            final Segment pSegment,
            final StringBuffer pStrBuff,
            final int indent)
    {
        for (int i = 0; i < pSegment.elementCount(); i++)
        {
            AbstractElement element = pSegment.getElement(i);

            format(element, pStrBuff, indent);
            if (i < pSegment.elementCount() - 1)
            {// nicht letzter Durchlauf
            }
        }
    }

    public static String format(final BracesArea pBracesArea, final int indent)
    {
        StringBuffer strBuff = new StringBuffer("\n" + StringUtil.spaces(indent) + "{\n");

        for (int i = 0; i < pBracesArea.elementCount(); i++)
        {
            Segment segment = pBracesArea.getSegment(i);

            format(segment, strBuff, indent + 1);
            if (!segment.isBracesSegment())
            {
                // kein Braces-Segment (geschwungene Klammern)
                // Semikolon als Trenner
                strBuff.append(";\n");
                if (i < pBracesArea.elementCount() - 1)
                {
                    // nicht letzter Durchlauf
                    strBuff.append("\n");
                }
            }
            // else
            // {
            // //geschwungene Klammern anfgen
            // strBuff = new StringBuffer( "\n" + StringUtil.spaces( indent ) + "{2\n" 
            // + strBuff + StringUtil.spaces( indent ) + "2}\n" ) ;
            // }
            // strBuff.append( "\n" );
        }
        strBuff.append(StringUtil.spaces(indent) + "}\n\n");
        return strBuff.toString();
    }

    public static String format(final ParenthesisArea pParenthesisArea, final int indent)
    {
        if (pParenthesisArea.isEmpty())
        {
            return "()";
        }
        StringBuffer strBuff = new StringBuffer(/* StringUtil.spaces( indent ) + */"( ");
        CommaSeparatedList commaSeparatedList = pParenthesisArea.getCommaSeparatedList();

        strBuff.append(format(commaSeparatedList, indent));
        strBuff.append(" )");
        return strBuff.toString();
    }

    public static String format(final BracketArea pBracketArea, final int indent)
    {
        if (pBracketArea.isEmpty())
        {
            return "[]";
        }
        StringBuffer strBuff = new StringBuffer(/* StringUtil.spaces( indent ) + */"[ ");
        CommaSeparatedList commaSeparatedList = pBracketArea.getCommaSeparatedList();

        strBuff.append(format(commaSeparatedList, indent));
        strBuff.append(" ]");
        return strBuff.toString();
    }

    public static String format(final CommaSeparatedList pCommaSeparatedList, final int indent)
    {
        if (pCommaSeparatedList == null)
        {
            return "";
        }
        StringBuffer strBuff = new StringBuffer();

        for (int i = 0; i < pCommaSeparatedList.elementListSize(); i++)
        {
            ElementList elementList = pCommaSeparatedList.getElement(i);
            String elemStr = format(elementList, indent + 1);

            strBuff.append(elemStr);
            if (i < pCommaSeparatedList.elementListSize() - 1)
            {
                // nicht letzter Durchlauf
                strBuff.append(" , ");
            }
        }
        return strBuff.toString();
    }

    public static String format(final ElementList pElementList, final int indent)
    {
        StringBuffer strBuff = new StringBuffer();

        for (int i = 0; i < pElementList.elementCount(); i++)
        {
            AbstractElement element = pElementList.getElement(i);

            format(element, strBuff, indent + 1);
            if (i < pElementList.elementCount() - 1)
            {
                // nicht letzter Durchlauf
                strBuff.append(" ");
            }
        }
        return strBuff.toString();
    }

    /**
     * @param element
     * @param i
     * @return
     */
    private static void format(
            final AbstractElement pElement,
            final StringBuffer pStrBuff,
            final int indent)
    {
        if (pStrBuff.toString().endsWith("\n"))
        {
            pStrBuff.append(StringUtil.spaces(indent));
        }
        else if (!pStrBuff.toString().endsWith(" "))
        {
            pStrBuff.append(" ");
        }

        if (pElement instanceof BracketComment)
        {
            pStrBuff.append(pElement.getContent() + "\n");
            return;
        }
        if (pElement instanceof InlineComment)
        {
            pStrBuff.append(pElement.getContent());
            return;
        }
        else if (pElement.isTerminal())
        {
            pStrBuff.append(pElement.getContent());
            return;
        }
        else if (pElement instanceof Segment)
        {
            format((Segment) pElement, pStrBuff, indent);
            return;
        }
        else if (pElement instanceof BracesArea)
        {
            pStrBuff.append(format((BracesArea) pElement, indent));
            return;
        }
        else if (pElement instanceof ParenthesisArea)
        {
            pStrBuff.append(format((ParenthesisArea) pElement, indent));
            return;
        }
        else if (pElement instanceof BracketArea)
        {
            pStrBuff.append(format((BracketArea) pElement, indent));
            return;
        }

        throw new UnsupportedOperationException("" + pElement.getClass().getName());

        // return StringUtil.spaces( indent  ) + pElement;
    }

}
