package de.cnc.domainparser.demo;


import java.util.HashMap;
import java.util.StringTokenizer;

import de.cnc.domainparser.BracesArea;
import de.cnc.domainparser.DomainParseException;
import de.cnc.domainparser.HighCommaStringLiteral;
import de.cnc.domainparser.Identifier;
import de.cnc.domainparser.Number;
import de.cnc.domainparser.QuoteStringLiteral;
import de.cnc.domainparser.Segment;
import de.cnc.domainparser.Unit;


/**
 * Diese Klasse dient zur Demonstration
 * der Verwendung des GenericParsers
 * als Ersatz fr properties- oder
 * XML-Files fr Konfigurationsdaten
 */
public class HierarchicProperties
{

    /** Werte HashMap */
    private HashMap hashMap = new HashMap();

    /**
     * 
     */
    public HierarchicProperties(Unit pUnit) 
        throws DomainParseException
    {
        super();
        pUnit.stripComments(true);
        pUnit.stripEmptySegments();

System.out.println( "" + pUnit );

        for (int i1 = 0; i1 < pUnit.segmentCount(); i1++)
        {
            readSegment(pUnit.getSegment(i1), this.hashMap);
        }        
    }

    /**
     * @param segment
     */
    private void readSegment(
            Segment pSegment,
            HashMap pHashMap)
        throws DomainParseException
    {
        if (pSegment.isBracesSegment())
        {
            if (pSegment.elementCount() != 2)
            {
                throw new DomainParseException("two elements expected", pSegment.getSourcePos());
            }
            if (!(pSegment.getElement(0) instanceof Identifier))
            {
                throw new DomainParseException("identifier expected",
                        pSegment.getElement(0).getSourcePos());
            }
            if (!(pSegment.getElement(1) instanceof BracesArea))
            {
                throw new DomainParseException("braces area expected",
                        pSegment.getElement(1).getSourcePos());
            }

            String keyStr = pSegment.getElement(0).getContent();
            HashMap recursivHashMap = new HashMap();

            if (pHashMap.get(keyStr) != null)
            {
                throw new RuntimeException("double (ambigous) element " + pSegment.getElement(0) + " at " + pSegment.getElement(0).getSourcePos() );
            }
            pHashMap.put(keyStr, recursivHashMap);

            for (int i = 0; i < ((BracesArea) (pSegment.getElement(1))).elementCount(); i++)
            {
                // call recursiv
                readSegment(((BracesArea) pSegment.getElement(1)).getSegment(i), recursivHashMap);
            }        
        }
        else
        {
            if (pSegment.elementCount() != 3)
            {
                throw new DomainParseException("three elements expected", pSegment.getSourcePos());
            }
            if (!(pSegment.getElement(0) instanceof Identifier))
            {
                throw new DomainParseException("identifier expected",
                        pSegment.getElement(0).getSourcePos());
            }
            if (!(pSegment.getElement(1).getContent() == "="))
            {
                throw new DomainParseException("= expected", pSegment.getElement(0).getSourcePos());
            }
            if (!(pSegment.getElement(2) instanceof Number
                    || pSegment.getElement(2) instanceof HighCommaStringLiteral
                    || pSegment.getElement(2) instanceof QuoteStringLiteral))
            {
                throw new DomainParseException("number or string literal expected",
                        pSegment.getElement(0).getSourcePos());
            }

            String keyStr = pSegment.getElement(0).getContent();
            Object valueObj = null;

            if (pSegment.getElement(2) instanceof Number)
            {
                valueObj = new Double(pSegment.getElement(2).getContent());
            }
            else
            {
                valueObj = pSegment.getElement(2).getContent();
            }
            pHashMap.put(keyStr, valueObj);
        }
    }

    /**
     * Abfragen eines Wertes.
     * Hierarchisch geschachtelte Werte
     * knnen durch Pfadangaben mit 
     * Schrgstrichen abgefragt werden.
     * 
     * @param pKey
     * @return
     */
    public String getValue(
            final String pKey)
    {
        HashMap tHashMap = this.hashMap;
        StringTokenizer stringTokenizer = new StringTokenizer(pKey, "/");
        Object valueObj = null;

        while (stringTokenizer.hasMoreTokens())
        {
            String subKeyStr = stringTokenizer.nextToken();

            valueObj = tHashMap.get(subKeyStr);
            if (stringTokenizer.hasMoreTokens())
            {
                if (valueObj instanceof HashMap)
                {
                    tHashMap = (HashMap) valueObj;
                }
                else
                {
                    return null;
                }
            }
        }
        if (valueObj instanceof String)
        {
            return (String) valueObj;
        }
        return null;
    }

    public String toString()
    {
        return getClass().getName() + ":\n" + this.hashMap.toString();
    }

}
