package de.cnc.domainparser.demo;


import java.util.ArrayList;
import java.util.List;

import de.cnc.domainparser.BracesArea;
import de.cnc.domainparser.DomainParseException;
import de.cnc.domainparser.DomainParserContext;
import de.cnc.domainparser.HighCommaStringLiteral;
import de.cnc.domainparser.Identifier;
import de.cnc.domainparser.Number;
import de.cnc.domainparser.QuoteStringLiteral;
import de.cnc.domainparser.Segment;
import de.cnc.domainparser.Unit;
import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse dient zur Demonstration
 * der Verwendung des GenericParsers
 * als Ersatz fr Properties- oder
 * XML-Files fr Konfigurationsdaten
 */
public class Yml
{

    //private static Logger log = Logger.getLogger(Yml.class);

    /** Einrckung fr Ausgabe (Dump) */
    private static final String INDENT = "  ";
    
    /**
     * Werte Root List .
     */
    private ArrayList rootList = new ArrayList();

    private String fileName;
    
    /**
     * @return Returns the rootList.
     */
    public ArrayList getRootList()
    {
        return this.rootList;
    }
    /**
     * 
     */
    public Yml(Unit pUnit) 
        throws DomainParseException
    {
        this.fileName = pUnit.getFileName();
        
        pUnit.stripComments(true);
        pUnit.stripEmptySegments();

        //System.out.println( "" + pUnit );

        for (int i1 = 0; i1 < pUnit.segmentCount(); i1++)
        {
            readSegment(pUnit.getSegment(i1), this.rootList);
        }        
    }

    /**
     * @param segment
     */
    private void readSegment(
            Segment pSegment,
            List pList)
        throws DomainParseException
    {
        if (pSegment.isBracesSegment())
        {
            if (pSegment.elementCount() != 2)
            {
                throw new DomainParseException("two elements (name and braces area) expected", pSegment.getSourcePos());
            }
            if (!(pSegment.getElement(0) instanceof Identifier))
            {
                throw new DomainParseException("identifier expected",
                        pSegment.getElement(0).getSourcePos());
            }
            if (!(pSegment.getElement(1) instanceof BracesArea))
            {
                throw new DomainParseException("braces area expected",
                        pSegment.getElement(1).getSourcePos());
            }

            final String keyStr = pSegment.getElement(0).getContent();

            final Element recursivElement = new Element( //--
                    keyStr , //--
                    pSegment.getElement(0).getSourcePos() );

            pList.add( recursivElement );
            
            for (int i = 0; i < ((BracesArea) (pSegment.getElement(1))).elementCount(); i++)
            {
                // call recursiv
                readSegment(((BracesArea) pSegment.getElement(1)).getSegment(i), //--
                        recursivElement.list );
            }        
        }
        else
        {
            if (pSegment.elementCount() /*!=*/< 3)
            {
                throw new DomainParseException( //--
                        "Yml-File: " + this.fileName + " " //--
                        + "three elements (Name = Value) expected", pSegment.getSourcePos());
            }
            if (!(pSegment.getElement(0) instanceof Identifier))
            {
                throw new DomainParseException(//--
                        "Yml-File: " + this.fileName + " " //--
                        + "identifier expected",
                        pSegment.getElement(0).getSourcePos());
            }
            if (!(pSegment.getElement(1).getContent() == "="))
            {
                throw new DomainParseException(//--
                        "Yml-File: " + this.fileName + " " //--
                        + "assign operator '=' expected", pSegment.getElement(0).getSourcePos());
            }
            if (!(pSegment.getElement(2) instanceof Number
                    || pSegment.getElement(2) instanceof HighCommaStringLiteral
                    || pSegment.getElement(2) instanceof QuoteStringLiteral
                    || pSegment.getElement(2).getNettoContent().equals( "true" ) 
                    || pSegment.getElement(2).getNettoContent().equals( "false" ) 
                    || pSegment.getElement(2).getNettoContent().equals( "null" ) 
                    // negatives Vorzeichen
                    || ( pSegment.elementCount() > 3 
                            && pSegment.getElement(2).getNettoContent().equals( "-" ) 
                            && pSegment.getElement(3) instanceof Number)
                    ) )
            {
                throw new DomainParseException(//--
                        "Yml-File: " + this.fileName + " " //--
                        + "number or string literal or 'true' or 'false' or 'null' expected >" //--
                        + pSegment.getElement(2).getNettoContent() + "<" , //--
                        pSegment.getElement(0).getSourcePos());
            }

            String keyStr = pSegment.getElement(0).getContent();
            Object valueObj = null;

            if (pSegment.getElement(2) instanceof Number)
            {
                valueObj = new Double(pSegment.getElement(2).getContent());
            }
            else if (pSegment.getElement(2).getNettoContent().equals( "-" ))
            {
                //negatives Vorzeichen
                valueObj = "-" + new Double(pSegment.getElement(3).getContent());
            }
            else if (pSegment.getElement(2).getNettoContent().equals( "null" ))
            {
                valueObj = null;
            }
            else if (pSegment.getElement(2).getNettoContent().equals( "true" ))
            {
                valueObj = Boolean.TRUE;
            }
            else if (pSegment.getElement(2).getNettoContent().equals( "false" ))
            {
                valueObj = Boolean.FALSE;
            }
            else if (pSegment.getElement(2) instanceof HighCommaStringLiteral //--
                        || pSegment.getElement(2) instanceof QuoteStringLiteral)
            {
                valueObj = pSegment.getElement(2).getNettoContent();
                
            }
            else
            {
                throw new RuntimeException(//--
                        "Yml-File: " + this.fileName + " " //--
                        + "program error, unreachable code");
            }
            Attribute attribute = new Attribute(//--
                    keyStr, //--
                    valueObj, //--
                    pSegment.getSourcePos() );

            pList.add( attribute );
        }
    }

//    /**
//     * Abfragen eines Wertes.
//     * Hierarchisch geschachtelte Werte
//     * knnen durch Pfadangaben mit 
//     * Schrgstrichen abgefragt werden.
//     * 
//     * @param pKey
//     * @return
//     */
//    public Object getValue(
//            final String pKey)
//    {
//        if ("".equals(pKey))
//        {
//            return this.rootMap;
//        }
//        Object tColl = this.rootMap;
//        StringTokenizer stringTokenizer = new StringTokenizer(pKey, "/");
//        Object valueObj = null;
//
//        while (stringTokenizer.hasMoreTokens())
//        {
//            String subKeyStr = stringTokenizer.nextToken();
//
//            if (subKeyStr.equals( "0" ) //--
//                    && ! (tColl instanceof List) //--
//                    && ! (tColl instanceof Map) )
//            {
//                //Index 0 passt immer
//            }
//            else if (tColl instanceof Map)
//            {
//                if ( onlyDigits( subKeyStr ) )
//                {
//                    //assign per number
//                    final int index = Integer.parseInt(subKeyStr);
//                    LinkedHashMap linkMap = (LinkedHashMap) tColl;
//                    tColl = linkMap.get( index );
//                }
//                else
//                {
//                    valueObj = ( (Map) tColl).get(subKeyStr);
//                    if (stringTokenizer.hasMoreTokens())
//                    {
//                        if (valueObj instanceof Map || valueObj instanceof List)
//                        {
//                            tColl = valueObj;
//                        }
//                        else
//                        {
//                            return null; // TODO exception
//                        }
//                    }
//                }
//            }
//            else if (tColl instanceof List)
//            {
//                final int index = Integer.parseInt(subKeyStr);
//                valueObj = ( (List) tColl).get(index);
//                if (stringTokenizer.hasMoreTokens())
//                {
//                    if (valueObj instanceof Map || valueObj instanceof List)
//                    {
//                        tColl = valueObj;
//                    }
//                    else
//                    {
//                        return null; // TODO exception
//                    }
//                }
//            }
//        }
////        if (valueObj instanceof String)
////        {
////            return (String) valueObj;
////        }
////        return null;
//        return valueObj;
//    }

    /**
     * 
     * @param pRootStr
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public int getValueCount(final String pRootStr)
//    {
//        Object value = getValue(pRootStr);
//        if (value == null)
//        {
//            return 0;
//        }
//        if (value instanceof Map)
//        {
//            //Map
//            return ( (Map) value).size();
//        }
//        if (value instanceof List)
//        {
//            //List
//            return ( (List) value).size();
//        }
//        //elementar value
//        return 1;
//    }

    /**
     * 
     * @param pRootStr
     * @param pI
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public Object getValue(//--
//            final String pRootStr, //--
//            final int pI)
//    {
//        Object parentValue = getValue(pRootStr);
//        if (parentValue == null)
//        {
//            return null; // TODO Exception
//        }
//        if (parentValue instanceof List)
//        {
//            //List
//            return ( (List) parentValue).get( pI );
//        }
//        else if (pI == 0)
//        {
//            //elementar value
//            return parentValue;
//        }
//        return null; // TODO Exception
//    }

    /**
     * 
     * @param pRootStr
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public boolean isMapOrList(String pRootStr)
//    {
//        Object parentValue = getValue(pRootStr);
//        if (parentValue == null)
//        {
//            return false; // TODO Exception
//        }
//        return (parentValue instanceof Map
//                || parentValue instanceof List);
//    }
    /**
     * 
     * @param pRootStr
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public boolean isList(String pRootStr)
//    {
//        Object parentValue = getValue(pRootStr);
//        if (parentValue == null)
//        {
//            return false; // TODO Exception
//        }
//        return (parentValue instanceof List);
//    }
    /**
     * 
     * @param pRootStr
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public boolean isMap(String pRootStr)
//    {
//        Object parentValue = getValue(pRootStr);
//        if (parentValue == null)
//        {
//            return false; // TODO Exception
//        }
//        return (parentValue instanceof Map);
//    }
    /**
     * 
     * @param pRootStr
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public Object[] getKeyArr(String pRootStr)
//    {
////System.out.println("getKeyArr: " + pRootStr);
//        Object parentValue = getValue(pRootStr);
////System.out.println("parentValue: " + parentValue);
//        
//        if (parentValue == null)
//        {
//            return null; // TODO Exception
//        }
//        if (parentValue instanceof Map)
//        {
//            Map parentMap = (Map) parentValue;
//            Object[] keyArr = parentMap.keySet().toArray();
//            return keyArr;
//        }
//        if (parentValue instanceof List)
//        {
//            //index numbers as keys
//            List list = (List) parentValue;
//            final int size = list.size();
//            /*Object*/ Integer[] keyArr = new /*Object*/ Integer[ size ];
//            for (int i = 0; i < size; i++)
//            {
//                keyArr[i] = new Integer( i );
//            }
//            return keyArr;
//        }
//        return null; // TODO Exception
//    }
    /**
     * 
     * @param pRootStr
     * @return
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
//    public String[] getKeyStrArr(String pRootStr)
//    {
////System.out.println("getKeyStrArr: " + pRootStr);
//        final Object[] keyObjArr = getKeyArr(pRootStr);
//        if (keyObjArr == null)
//        {
//            return null;
//        }
//        final String[] keyStrArr = new String[ keyObjArr.length ];
//        if ( keyObjArr instanceof Integer[] )
//        {
//            //his was a List, dirty trick with Integer[]
//            for (int i = 0; i < keyObjArr.length; i++)
//            {
//                keyStrArr[ i ] = keyObjArr[ i ].toString();
//            }
//        }
//        else
//        {
//            System.arraycopy(keyObjArr,0,keyStrArr,0,keyObjArr.length);
//        }
//        return keyStrArr;
//    }

    /**
     * Debug.
     */
    public String toString()
    {
        return getClass().getName() + ":\n" + this.rootList;
    }

    public static final String YML_IN_FILE = "./src/de/cnc/domainparser/test.yml";

    /**
     * 
     * 
     * @param pYml
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
    public /*static*/ void dump()//--
            //final Yml pYml)
    {
        //recurDump(pYml.rootList, "");
        recurDump(this.rootList, "");
    }
    
    /**
     * 
     * 
     * @param pYml
     * @param pIndent
     * 
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
    public static void recurDump(//--
            final List pList, //--
            final String pIndent)
    {
        for (int i = 0; i < pList.size(); i++)
        {
            Object element = pList.get( i );
            if (element instanceof Attribute)
            {
                Attribute attribute = (Attribute) element;
                final Object value = attribute.value;
                final String valueStr;
                if (value instanceof String)
                {
                    //valueStr = "'" + valueStr + "'";
                    valueStr = StringUtil.encodeToJavaStrLiteral( (String) value );
                }
                else
                {
                    valueStr = "" + value;
                }
                System.out.println( pIndent + attribute.name + " = " + valueStr + ";" );
            }
            else if (element instanceof Element)
            {
                Element subElem = (Element) element;
                System.out.println(  pIndent + subElem.name + " {" );
                recurDump( subElem.list , pIndent + INDENT );
                System.out.println(  pIndent + "}" );
            }
            else
            {
                throw new RuntimeException("unreachable code");
            }
        }
    }

    
    
    
    /**
     * 
     * 
     * @param pYml
     * @param pRootStr
     * @param pIndent
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
/*
    public static void recurDump(//--
            final List pList, //--
            final String pRootStr, //--
            final String pIndent)
    {
            final int valueCount = pYml.getValueCount(pRootStr);
            for (int i = 0; i < valueCount; i++)
            {
                if (pYml.isMapOrList(pRootStr + "/" + i))
                {
                    System.out.println( pIndent + i + " " + ( pYml.isMap(pRootStr + "/" + i) ? '{' : '[' ) );
                    recurDump(pYml, pRootStr + "/" + i, pIndent + " " );
                    System.out.println( pIndent + ( pYml.isMap(pRootStr + "/" + i) ? '}' : ']' ) );
                }
                else
                {
                    final Object subValue = pYml.getValue(pRootStr, i);
                    System.out.println( pIndent + ( i ) + " = " + subValue + ';' );
                }
            }
        }
        else if (pYml.isMap(pRootStr))
        {
            Object[] keyArr = pYml.getKeyArr(pRootStr);
            for (int i = 0; i < keyArr.length; i++)
            {
                Object key = keyArr[i];
                if (pYml.isMapOrList(pRootStr + "/" + key))
                {
                    System.out.println( pIndent + keyArr[ i ] + " " + ( pYml.isMap(pRootStr + "/" + keyArr[ i ]) ? '{' : '[' ) );
                    recurDump(pYml, pRootStr + "/" + keyArr[ i ], pIndent + " " );
                    System.out.println( pIndent + ( pYml.isMap(pRootStr + "/" + keyArr[ i ]) ? '}' : ']' ) );
                }
                else
                {
                    final Object subValue = pYml.getValue(pRootStr + "/" + key);
                    System.out.println( pIndent + ( key ) + " = " + subValue + ';' );
                }
            }
            
        }
        else
        {
            //throw new IllegalArgumentException(pRootStr);
            System.out.println( pIndent + pYml.getValue(pRootStr) );
        }
    }
*/
    
    
    public static class Attribute
    {
        public final String name;
        public final Object value;
        public final SourcePosition srcPos;
        
        /**
         * Constructor .
         * 
         * @param pName
         * @param pValue
         */
        public Attribute(//--
                final String pName, //--
                final Object pValue, //--
                final SourcePosition pSrcPos)
        {
            if (pName == null)
            {
                throw new NullPointerException();
            }
            //if (pValue == null)
            //{
            // value can be null
            //    throw new NullPointerException();
            //}
            if (pSrcPos == null)
            {
                throw new NullPointerException();
            }
            this.name = pName;
            this.value = pValue;
            this.srcPos = pSrcPos;
        }
        /**
         * Debug.
         */
        public String toString()
        {
            return this.getClass().getName() + "@" + Integer.toHexString(this.hashCode()) + "[" 
            + "name: " + this.name + ", "
            + "value" + this.value + "]";
        }
    }
    
    public static class Element
    {
        public final String name;
        public final ArrayList list = new ArrayList();
        public final SourcePosition srcPos;

        /**
         * Constructor .
         * 
         * @param pName
         */
        public Element(//--
                final String pName, //--
                final SourcePosition pSrcPos)
        {
            if (pName == null)
            {
                throw new NullPointerException();
            }
            if (pSrcPos == null)
            {
                throw new NullPointerException();
            }
            this.name = pName;
            this.srcPos = pSrcPos;
        }
        /**
         * Debug.
         */
        public String toString()
        {
            return this.getClass().getName() + "@" + Integer.toHexString(this.hashCode()) + "[" 
            + "name: " + this.name + ", "
            + "list" + this.list + "]";
        }
    }

    /**
     * Test.
     * 
     * @param args
     * @author Heiner Kcker
     * @since Jun 23, 2005
     */
    public static void main(String[] args)
    {
        System.out.println("---------------------------------------------------------");
        System.out.println("Test Yml");

        String inputStr = StringUtil.file2str(YML_IN_FILE);

        System.out.println("inputStr:\n" + inputStr);

        System.out.println("---------------------------------------------------------");
        System.out.println("Test Yml geparst");

        final DomainParserContext parserContext = new DomainParserContext();
        parserContext.isJavaStringEncodingEnabled = true;
        parserContext.isNewlineInStringEnabled = true;
        Unit tUnit = null;

        try
        {
            tUnit = Unit.parseStr(inputStr, YML_IN_FILE, parserContext);
        }
        catch (DomainParseException e)
        {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
        System.out.println("---------------------------------------------------------");

        System.out.println("" + tUnit);

        System.out.println("---------------------------------------------------------");

        Yml yml = null;

        try
        {
            yml = new Yml(tUnit);
        }
        catch (DomainParseException e)
        {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
        System.out.println("" + yml);
        System.out.println("---------------------------------------------------------");
        System.out.println("Test Yml Abfrage Werte");

        yml.dump();

        //System.out.println("c1/d/1/c.c.c: " + yml.getValue("c1/d/1/c.c.c"));
        
        System.out.println("---------------------------------------------------------");

    }
    
}
