package de.cnc.domainparser.util;

import de.cnc.util.CompareUtil;
import de.cnc.util.StringUtil;


/**
 * Position im Source-Code bestehend
 * aus Zeile, Spalte und Absolut-Position
 * 
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 * 
 * @author Heiner Kcker
 * @version $Id: SourcePosition.java,v 1.3 2005/07/11 11:15:13 hku Exp $
 */
public class SourcePosition
{

    /**
     * Absolutposition ohne Beachtung Zeilen und Spalten
     * Basis 0
     */
    private int pos;

    /**
     * Zeilenposition Basis 1
     */
    private int row = 1;

    /**
     * Spaltenposition Basis 1
     */
    private int col = 1;

    private String fileName;

    /**
     * parameterloser Konstruktor
     */
    private SourcePosition()
    {
        ;
    }

    /**
     * parameterbehafteter Konstruktor
     */
    public SourcePosition(//--
            final String pFileName)
    {
        this.fileName = pFileName;
    }

    /**
     * parameterbehafteter Konstruktor
     */
    public SourcePosition(//--
            final int paPos, //--
            final int paRow, //--
            final int paCol, //--
            final String pFileName)
    {
        if (paPos < 0)
        {
            throw new RuntimeException("pos must >= 0");
        }

        if (paRow < 1)
        {
            throw new RuntimeException("row must > 0");
        }

        if (paCol < 1)
        {
            throw new RuntimeException("col must > 0");
        }

        if (paRow == 0 && paPos != paCol)
        {
            throw new RuntimeException("pos must equals to col if row == 0");
        }

        if (paRow > 1 && paCol < 2)
        {
            throw new RuntimeException("pos + 1 must equals to col if row == 0");
        }

        this.pos = paPos;
        this.row = paRow;
        this.col = paCol;
        this.fileName = pFileName;
    }

    /**
     * addiere die bergebene Anzahl Zeichen zur SourcePosition,
     * aber kein Newline.
     * 
     * @param iPaSteps zu addierende Anzahl Zeichen
     */
    public SourcePosition add(int iPaSteps)
    {
        this.pos += iPaSteps;
        this.col += iPaSteps;
        return this;
    }

    /**
     * addiere ein Zeichen zur SourcePosition,
     * aber kein Newline.
     */
    public SourcePosition addOne()
    {
        this.pos++;
        this.col++;
        return this;
    }

    /**
     * Addiere ein Newline zur SourcePosition,
     * dabei wird die Spalte auf 0 gesetzt und
     * die Absolutposition und die Zeilenposition
     * erhht.
     */
    public SourcePosition addNewLine()
    {
        this.pos++;
        this.row++;
        this.col = 1;
        return this;
    }

    /**
     * gibt einen Clone der SourcePosition zurck.
     * 
     * @return geclonte SourcePosition
     */
    public SourcePosition getClone()
    {
        SourcePosition retSourcePosition = new SourcePosition();

        retSourcePosition.pos = this.pos;
        retSourcePosition.col = this.col;
        retSourcePosition.row = this.row;
        retSourcePosition.fileName = this.fileName;
        return retSourcePosition;
    }

   /**
    * Setzt die Positionswerte anhand der bergebenen Position
    * 
    * @return geclonte SourcePosition
    */
   public void set(final SourcePosition pPos)
   {
       this.pos = pPos.pos;
       this.col = pPos.col;
       this.row = pPos.row;
       this.fileName = pPos.fileName;
   }

    /**
     * Gets the col
     * 
     * @return Returns a int
     */
    public int getCol()
    {
        return this.col;
    }

    /**
     * Gets the pos
     * 
     * @return Returns a int
     */
    public int getPos()
    {
        return this.pos;
    }

    /**
     * Gets the row
     * 
     * @return Returns a int
     */
    public int getRow()
    {
        return this.row;
    }

    /**
     * @return Returns the fileName.
     */
    public String getFileName()
    {
        return this.fileName;
    }

    /**
     * debug output
     */
    public String toString()
    {
        String strRet = //--
            StringUtil.getShortClassName(this) //--
                + " pos: " + this.pos //--
                + " row: " + this.row //--
                + " col: " + this.col //--
                + " fileName: " + this.fileName;

        return strRet;
    }// end method

    /**
     * Prft, ob das bergebene Objekt eine identische SourcePosition ist.
     *
     * @see Object#equals(Object)
     * @param other
     */
    public boolean equals(Object other)
    {
        if (other == null)
        {
            return false;
        }

        if (other.getClass().equals(this.getClass()))
        {
            return //--
                CompareUtil.isEqual( this.fileName , ((SourcePosition) other).fileName )
                    && this.pos == ((SourcePosition) other).pos
                    && this.row == ((SourcePosition) other).row
                    && this.col == ((SourcePosition) other).col;
        }

        return false;
    }

}

